#Class of 'wires'. 
#Each connection through localProxy is a wire.
#Implements connection objects.
package Wires;
use strict;
use Carp;

my $debugging = 0;	#file-scoped lexical
our $AUTOLOAD;  		#package global

my $errorInit = 1000; #XXX get from caller

my %fields = (
	serviceId   => undef,
	commStrat   => undef,
	layer0Index => undef,
	layer1Index => undef,
	insideSocketHandle => undef,
	outsideSocketHandle => undef,
	outsideSocketServiceSubtype => undef, #for http/1.1, or 1.0
	state => undef,
	stats => {
		vol => 0, 
		time => 0, 
		error => $errorInit, 
		totalBytesIncoming => undef,
		lastInAt => undef,
		totalBytesOutgoing => undef,
		lastOutAt => undef,
		}, 
	);

sub new {
	my $that  = shift;
	my $class = ref($that) || $that;
	my $self  = {
		_permitted => \%fields,
		%fields,
		};
	bless $self, $class;
	return $self;
	}
	
sub AUTOLOAD {
	my $self = shift;
	my $type = ref($self)
		or croak "$self is not an object";
	my $name = $AUTOLOAD;
	$name =~ s/.*://;   # strip fully-qualified portion
	unless (exists $self->{_permitted}->{$name} ) {
		croak "Can't access `$name' field in class $type";
		}
	if (@_) {
		return $self->{$name} = shift;
		} 
	else {
		return $self->{$name};
		}
	}
	
sub otherEnd { #return handle for socket at other end
	my ($self, $sh) = @_;
	if ($sh == $self->{insideSocketHandle}) {
		return $self->{outsideSocketHandle}
		}
	elsif ($sh == $self->{outsideSocketHandle}) {
		return $self->{insideSocketHandle}
		}
	else {warn "wrong wire called for $sh"; return undef}
	};
		
	
sub debug {
	my $self = shift;
	confess "usage: thing->debug(level)"    unless @_ == 1;
	my $level = shift;
	if (ref($self))  {
		$self->{"_DEBUG"} = $level;         # just myself
		} 
	else {
		$debugging        = $level;         # whole class
		}
	}
	
sub DESTROY {
	my $self = shift;
	if ($debugging || $self->{"_DEBUG"}) {
		carp "Destroying $self " . $self->name;
		}
	}

1;  # so the require or use succeeds
